#! /bin/sh

ZIPARCHIV="$1"
test -f "$ZIPARCHIV" || stat "$ZIPARCHIV" || exit 2

# Delete current source files
rm -rf nedit-ng-master

# Unpack new source files
unzip -q "$ZIPARCHIV"

# Timestamp of the last modified source file
newestfile=$(find nedit-ng-master -type f -name "*.cpp" -o -name "*.h" | xargs ls --sort=t 2>/dev/null | head -1)
package_date=$(stat -c '%Y' $newestfile)

echo "Last modified file:"
printf "\t%s\t(%s)\n" "$newestfile"  "$(date --date=@$package_date)"

# Nedit-NG version and revision
nedit_version=$(egrep '(NEDIT_VERSION_MAJ|NEDIT_VERSION_REV) =' nedit-ng-master/Util/include/Util/version.h \
	| sed 's/.*\(NEDIT_VERSION.*\) = \([0-9]*\);/\1=\2/')

eval $nedit_version
printf "Nedit-NG Version:\t%i.%i\n" $NEDIT_VERSION_MAJ $NEDIT_VERSION_REV

PACKAGE_VERSION=$(date --date=@$package_date +'%Y%m%d')

# Append debian/changelog
#	Note: Debian's dpkg-buildpackage sets the timestamps of package contents according to changelog date.
#	(https://wiki.debian.org/ReproducibleBuilds)
{
	printf "nedit-ng (1:%i.%i-%i) UNRELEASED; urgency=low\n" $NEDIT_VERSION_MAJ $NEDIT_VERSION_REV $PACKAGE_VERSION
	printf "\n  * Package rebuilt (%s)\n\n" $(date +'%Y/%m/%d')
	printf " -- %s <%s@%s>  %s\n\n" $(id -un) $(id -un) $(hostname) "$(LC_ALL='' LANG=POSIX date --date=@$package_date --rfc-2822)"
    cat debian/changelog
} >> debian/changelog_n
mv debian/changelog_n debian/changelog


# Apply patches
patch -p0  < debian/patches/package-version.patch

# Set PACKAGE_VERSION
f='nedit-ng-master/Util/include/Util/version.h'
sed 's/12345678/'$PACKAGE_VERSION'/' ${f} > ${f}_n
mv ${f}_n ${f}
